@@ -11,11 +11,14 @@
 # See the "INSTALL.WEB" file for HTML fragments of files used by the
 # script (wait_t, done_t, and the main form itself), and crontab entries.
 # -----------------------------------------------------------------------
-my $VERSION = "3.00";
+my $VERSION = "3.21";
 # Last Major Modification: October 21, 2004
 # Changed aczone to aczoom
 # Updates by Avinash Chopde <avinash@acm.org> http://www.aczoom.com/
 # Updated: Aug 2007: added support for barcodegen using --tray-overlay
+# Updated: Oct 2008: added support for double width DVDs --double-case
+# Updated: Dec 2008: add -sPAPERSIZE to ps2pdf also, for correct A4 size
+# Updated: Nov 2009: add nocoverheading option
 # -----------------------------------------------------------------------
 # Created: March 2001, by Avinash Chopde <avinash@acm.org>  www.aczoom.com
 # -----------------------------------------------------------------------
@@ -51,12 +54,22 @@ my $VERSION = "3.00";
 # March 2001, Avinash Chopde <avinash@acm.org> http://www.aczoom.com/
 #------------------------------
 # Rough outline of things needed to make this work at your site:
-# Programs needed, on Linux: netpbm, 
+# Programs needed, on Linux:
+# sudo apt-get install netpbm
 # GhostScript - use version 5.10 - it is twice as fast as version 6.5
 # cdlabelgen - install (default goes in /usr/local/bin and /usr/local/lib)
 # netpbm - using version 9.22 (Dec 2001)
 # jpeg2ps - using version 1.8, from http://www.pdflib.com/jpeg2ps/
 # CGI.pm - latest one - 2.752 or newer
+# jpegtran: sudo apt-get install libjpeg-progs 
+# sudo cpan -i HTML::FillInForm
+# ----
+# barcodegen - on Ubuntu 9.04, this works:
+# sudo apt-get install imagemagick
+# sudo apt-get install libgd-tools
+# sudo apt-get install libgd2-xpm-dev
+# sudo apt-get install perlmagick
+# sudo cpan -i GD::Barcode::Image
 # ----
 # Make this folder non-writeable:
 # $ROOTDIR 
@@ -89,7 +102,11 @@ my $VERSION = "3.00";
 # -t <filename> should use full path for filename.
 #-----------------------------------------------------------------------------
 use 5.005; # perl newer than 5.005 required
-use CGI 2.50 qw(escapeHTML); # 2.47 for upload, 2.50 for Vars
+#-----------------------------------------------------------------------------
+use CGI 3.21 qw(escapeHTML); # 2.47 for upload, 2.50 for Vars, 3.21 for POST_MAX fix
+$CGI::POST_MAX=1024 * 800;  # max size posts accepted, bytes
+#-----------------------------------------------------------------------------
+
 use CGI::Carp qw(fatalsToBrowser);
 use Getopt::Std;
 use POSIX qw(floor);
@@ -97,10 +114,28 @@ use File::Copy;
 use Socket qw(:DEFAULT :crlf);
 use HTML::Template;
 use HTML::FillInForm;
+
+$start_time = time();
+$datestr = localtime($start_time);
 #-----------------------------------------------------------------------------
-$CGI::POST_MAX=1024 * 600;  # max size posts accepted, bytes
+$LOGFILE = "/home/cgi/tmp/weblog.txt";
+# log file - global script issues, not related to any one invocation
+# make sure this file exists, and is chmod 662 - writeable, but not readable
+# by others so casual web hackers can't read this directly using a browser.
 # Note that the PDF file generated is around 4 times image sizes!
 
+open(LOGFILE, ">> $LOGFILE")  
+  or myexit("Could not open log file ($LOGFILE): $!");
+select((select(LOGFILE), $| = 1)[0]); # autoflush
+
+# all STDERR output is caught in local log file, so any perl errors
+# or uncaught STDERR from commands goes here, instead of the global
+# apache log file
+open (STDERR, '>>& LOGFILE')
+  or myexit("Could not redirect STDERR to log file ($LOGFILE): $!");
+
+print LOGFILE "cdinsert [$datestr] $ENV{REMOTE_ADDR} - before new CGI - $ENV{REQUEST_METHOD}\n";
+
 $ENV{'PATH'}="/bin:/usr/bin:/usr/local/bin:/usr/local/netpbm/bin:/home/cgi/bin"; # security blanket (make sure all folders/files are non-writeable by others!)
 
 # Following vars need to be set specifically for each site 
@@ -110,11 +145,6 @@ $ROOTDIR = "/home/cgi/cdinsert"; # where all web files are kept
 # the HTTP address is:
 # $ROOTHTTP = "/cgi/cdinsert"; # relative URL, absolute path
 
-$LOGFILE = "/home/cgi/tmp/weblog.txt";
-# log file - global script issues, not related to any one invocation
-# make sure this file exists, and is chmod 662 - writeable, but not readable
-# by others so casual web hackers can't read this directly using a browser.
-
 # for each invocation - semi-random name for files - gets some privacy, since
 # these folders and files are readable by the world.
 $WORKID = floor(rand(1e4)); 
@@ -128,6 +158,7 @@ $SAVEFORM = "form" . $WORKID; # save form data here
 $INITIAL_TEMPLATE="initial.tmpl"; # first page, diplayed with form
 
 $FORM_TEMPLATE="form.tmpl"; # just the form
+$FORM_ACTION="/cgi-bin/cdinsert.pl"; # FORM ACTION="..." filled in with this
 
 $WORKING_TEMPLATE="working.tmpl"; # "working..." display template - this is
 # the HTML output of this script  to stdout, while it is doing the processing
@@ -166,19 +197,29 @@ $form_html = ""; # filled in form, displayed on results page
 select((select(STDOUT), $| = 1)[0]); # autoflush
 
 $query = new CGI;
+# above line prints warning: Use of uninitialized value in substitution (s///) at (eval 8) line 23. But webitrans.pl does not get this error... strange
+# started after updated perl with yum, and CGI.pm with CPAN to 3.35
 
 # output HTML page headers, rest of HTML is output by copying WORKING_TEMPLATE
-print CGI::header();
+print CGI::header(-charset => "ISO-8859-1");
 
 getopts('t:'); # -t <filename> is test mode. Filename is used as $incontents
 
 # --------------------------------
 # check if we need to do any work
+$cgierror = $query->cgi_error(); # post too big, or user hit STOP, etc...
+
+$form_tmpl = HTML::Template->new(filename => $FORM_TEMPLATE,
+                                 path => [$ROOTDIR]);
+$form_tmpl->param(MRKR_FORM_ACTION => $FORM_ACTION);
+$form_html = $form_tmpl->output();
+
 
 # if called without any form data (or -t test option), then just display
 # a page with the form and exit
-unless ($opt_t || $query->param()) {
+unless ($cgierror || $opt_t || $query->param()) {
   my $tmpl = HTML::Template->new(filename => $INITIAL_TEMPLATE, path => [$ROOTDIR]);
+  $tmpl->param(MRKR_FORM_HTML => $form_html);
   print $tmpl->output();
   exit(0);
 }
@@ -186,8 +227,6 @@ unless ($opt_t || $query->param()) {
 #-----------------------------------------------------------------------------
 # start work, create work directory, etc
 
-$start_time = time();
-
 $SIG{HUP} = $SIG{INT} = $SIG{QUIT} = $SIG{PIPE} = $SIG{TERM} = \&sighandler;
 
 # for each invocation - semi-random name - gets some privacy, since
@@ -216,7 +255,7 @@ $REDIRECT="< /dev/null 2>&1";
 # -- cmd 1:
 $CDLBL_E = "cdlabelgen $REDIRECT";
 # -- cmd 2:
-$PS2PDF_E = "ps2pdf $WORKFILE.ps $REDIRECT";
+$PS2PDF_E = "ps2pdf -sPAPERSIZE=letter $WORKFILE.ps $REDIRECT";
 # -- cmd 3:
 $PS2PGM_E = "gs -q -dNOPAUSE -dBATCH -dTextAlphaBits=4 -dGraphicsAlphaBits=4 -r72 -sDEVICE=pgmraw -sPAPERSIZE=letter -sOutputFile=$WORKFILE.pgm -f $WORKFILE.ps $REDIRECT";
 # gs options:
@@ -243,16 +282,6 @@ $BCODE_E = "barcodegen --format=EPS --border=7 --write=$BCODEFILE.eps $REDIRECT"
 $CDL_BCODE_ARGS="--tray-overlay $BCODEFILE.eps --tray-overlay-scaleratio=1,-0.1,0.1";
 
 #-----------------------------------------------------------------------------
-open(LOGFILE, ">> $LOGFILE")  
-  or myexit("Could not open log file ($LOGFILE): $!");
-select((select(LOGFILE), $| = 1)[0]); # autoflush
-
-# all STDERR output is caught in local log file, so any perl errors
-# or uncaught STDERR from commands goes here, instead of the global
-# apache log file
-open (STDERR, '>>& LOGFILE')
-  or myexit("Could not redirect STDERR to log file ($LOGFILE): $!");
-
 umask(0);
 chdir $WORKDIR or myexit("cd ($WORKDIR) failed: $!");
 open(MSGFILE, ">> $MSGFILE")  
@@ -267,7 +296,6 @@ open(SAVEFORM, ">> $SAVEFORM")
 open(WORKING_TEMPLATE, "< $ROOTDIR/$WORKING_TEMPLATE")
   or myexit("Could not open WORKING_TEMPLATE file ($WORKING_TEMPLATE): $!");
 
-$datestr = localtime($start_time);
 print MSGFILE "[$datestr] [pid $$]: Starting job in directory $WORKDIRNAME\n";
 
 # ----------------
@@ -322,6 +350,7 @@ if (! $opt_t ) {
     $incontents =~ s/$CR?$LF/\n/g; # fix all CR/LF chars
     $incdcase = $query->param('cdcase') || 'normal';
     $innotrayhd = $query->param('notrayheading') || '';
+    $innocoverhd = $query->param('nocoverheading') || '';
     $inscaleitems = $query->param('scaleitems') || '';
     $inmakegif = $query->param('makegif') || '';
     $ina4paper = $query->param('a4paper') || '';
@@ -345,6 +374,7 @@ if (! $opt_t ) {
     $intimagefile = '';
     $incdcase = 'normal';
     $innotrayhd = 0;
+    $innocoverhd = 0;
     $inscaleitems = 0;
     $inmakegif = 1;
     $insplititems = 0;
@@ -370,12 +400,10 @@ print LOGFILE "$WORKDIRNAME cdinsert [$datestr] $hostname - started\n";
 #----------------------------------------------------------------
 
 print MSGFILE "[$datestr] checking cgi_error...\n";
-$cgierror = $query->cgi_error(); # post too big, or user hit STOP, etc...
-# in some rare cases, cgi_error() takes over 2 minutes, not sure what
-# is wrong, probably a remote browser problem?
+
 if ($cgierror) {
     if ($cgierror =~ /413/) {
-	myexit("Received too much data, can receive max of " . int($CGI::POST_MAX/1024) . "KBytes.");
+	myexit("Uploaded image files too large?<br/> Received too much data - got <b>" . int($ENV{'CONTENT_LENGTH'}/1024) . "</b> KBytes, can only receive maximum of <b>" . int($CGI::POST_MAX/1024) . "</b> KBytes.");
     } else {
 	myexit($cgierror);
     }
@@ -500,6 +528,8 @@ $incdcase =~ /([\d\w\-\.]*)/; # no / allowed in name
 $incdcase = $1;
 $innotrayhd =~ /([\d\w\-\.]*)/; # no / allowed in name
 $innotrayhd = $1;
+$innocoverhd =~ /([\d\w\-\.]*)/; # no / allowed in name
+$innocoverhd = $1;
 $inscaleitems =~ /([\d\w\-\.]*)/; # no / allowed in name
 
 # Title and Subtitle should use entire string as entered by user
@@ -573,8 +603,9 @@ if ($incdcase =~ /^normal/) {
 # ---- compute page offset for A4 and gs command modifications
 if ($ina4paper) {
   $PS2PGM_E =~ s/PAPERSIZE=letter/PAPERSIZE=a4/;
+  $PS2PDF_E =~ s/PAPERSIZE=letter/PAPERSIZE=a4/;
   $ina4paper = "-y 1.5"; # default
-  $ina4paper = "-y 0.8" if ($incdcase =~ /^(dvd)|(envelope)/);
+  $ina4paper = "-y 0.8" if ($incdcase =~ /^(dvd)|(envelope)|(double)/);
 }
 #-----------------------------------------------------------------------------
 # check if barcode generation has to be performed - this requires
@@ -620,8 +651,10 @@ push(@cmdargs, "-m") if ($incdcase =~ /^slimcase/);
 push(@cmdargs, "-M") if ($incdcase =~ /^envelope/);
 push(@cmdargs, "--create-dvd-inside") if ($incdcase =~ /^dvdinside/);
 push(@cmdargs, "--create-dvd-outside") if ($incdcase =~ /^dvdoutside/);
+push(@cmdargs, "--double-case") if ($incdcase =~ /^doublecase/);
 push(@cmdargs, "-p") if (! $inscaleitems); 
 push(@cmdargs, "-b") if ($innotrayhd);
+push(@cmdargs, "-C") if ($innocoverhd);
 push(@cmdargs, $ina4paper) if ($ina4paper);
 push(@cmdargs, $CDL_BCODE_ARGS) if (-s "$BCODEFILE.eps");
 
@@ -686,8 +719,6 @@ print MSGFILE "----------------------------------------------------\n";
 # copy output HTML page show output or error, as appropriate
 
 # add the form to the done file, with filled in values
-$form_tmpl = HTML::Template->new(filename => $FORM_TEMPLATE, path => [$ROOTDIR]);
-$form_html = $form_tmpl->output();
 $form_fillin = HTML::FillInForm->new();
 $form_html = $form_fillin->fill(scalarref => \$form_html, fobject => $query);
 
@@ -728,23 +759,22 @@ sub replace_markers {
     my($tmpl) = @_;
     # apply to only WORKING_TEMPLATE
     $tmpl->param(MRKR_DONTREDIRECT => $dontredirect);
+    $tmpl->param(MRKR_WEBDONEHTML => "$WORKHTTP/$WEBDONE_HTML");
 
     # apply to both WORKING_TEMPLATE and DONE_TEMPLATE
-    $tmpl->param(MRKR_WEBDONEHTML => "$WORKHTTP/$WEBDONE_HTML");
     $tmpl->param(MRKR_HOSTNAME => "$hostname ( $hostaddr )");
     $tmpl->param(MRKR_ANYERROR => $errflag);
     $tmpl->param(MRKR_MSGFILE => "$WORKHTTP/$MSGFILENAME");
 
     # apply to only DONE_TEMPLATE
     $tmpl->param(MRKR_INPUT => "$WORKHTTP/$SAVINPUT");
-    $tmpl->param(MRKR_PDF => "$WORKHTTP/$WORKFILE.pdf");
+    $tmpl->param(MRKR_PDF => "$WORKHTTP/$WORKFILE.pdf") if (-s "$WORKFILE.pdf");
     $tmpl->param(MRKR_MAKEGIF => $inmakegif);
-    $tmpl->param(MRKR_MAKEBCODE => $inbcodetext) if (-s "$BCODEFILE.eps");
-    $tmpl->param(MRKR_GIF => "$WORKHTTP/$WORKFILE.gif");
-    $tmpl->param(MRKR_POSTSCRIPT => "$WORKHTTP/$WORKFILE.ps");
+    $tmpl->param(MRKR_GIF => "$WORKHTTP/$WORKFILE.gif") if (-s "$WORKFILE.gif");
+    $tmpl->param(MRKR_POSTSCRIPT => "$WORKHTTP/$WORKFILE.ps") if (-s "$WORKFILE.ps");
     $tmpl->param(MRKR_TIMETAKEN => $time_taken_units);
     $tmpl->param(MRKR_FORM_HTML => $form_html);
-    $tmpl->param(MRKR_BCODE_EPS => "$WORKHTTP/$BCODEFILE.eps");
+    $tmpl->param(MRKR_BCODE_EPS => "$WORKHTTP/$BCODEFILE.eps") if ($inbcodetext && -s "$BCODEFILE.eps");
 
     if ($errflag) {
       if ($errcmd =~ /$CDLBL_E/) {
